@@ -5,7 +5,7 @@
 #include "config.h"
 #include "meta.h"
 
-#define ACVERSION "0.2.2"
+#define ACVERSION "0.3.4"
 
 class hostdata;
 
@@ -13,10 +13,10 @@ namespace acfg
 {
 
 extern MYSTD::string cachedir, logdir, confdir, fifopath, user, group, pidfile, 
-reportpage, vfilepat, pfilepat, agentname, remoteport, adminauth, bindaddr, port;
+reportpage, vfilepat, pfilepat, wfilepat, agentname, remoteport, adminauth, bindaddr, port, sUmask;
 
 extern int debug, offlinemode, foreground, verbose, stupidfs, forcemanaged, 
-verboselog, extreshhold, tpoolsize, dnscachetime, dlbufsize;
+verboselog, extreshhold, exfailabort, tpoolsize, dnscachetime, dlbufsize, exporigin, logxff;
 
 extern tHttpUrl proxy_info;
 
@@ -30,7 +30,7 @@ void PostProcConfig();
 // throw away the rewritten part of the path, foo/debian/bla.deb -> bla.deb,
 // no slash needed with backends...
 				
-const MYSTD::string * GetRepNameAndPathResidual(const tHttpUrl in, MYSTD::string & sRetPathResidual);
+const MYSTD::string * GetRepNameAndPathResidual(const tHttpUrl & in, MYSTD::string & sRetPathResidual);
 
 typedef MYSTD::vector<tHttpUrl> tHostiVec; 
 tHostiVec * GetBackendVec(const MYSTD::string * vname);
