@@ -11,6 +11,7 @@ namespace aclog {
       void reopen();
       void transfer(bool bDirIn, uint64_t nCount, const char *szClient, const char *szPath);
       void err(const char *msg, const char *client=NULL);
+      void misc(const MYSTD::string & sLine);
       inline void err(const MYSTD::string msg) { err(msg.c_str()); }
       void flush();
       
@@ -21,6 +22,11 @@ namespace aclog {
       } tRowData;
 
       void GetStats(MYSTD::vector<tRowData> & out);
+      
+      struct errnoFmter {
+    	  errnoFmter();
+    	  const char *msg;
+      };
 }
 
 #define TIMEFORMAT "%a %d/%m"
