@@ -2,9 +2,31 @@
 #define CADDRINFO_H_
 
 #include "meta.h"
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <netdb.h>
+#include "rfc2553emu.h"
+ 
+#ifndef HAVE_GETADDRINFO
+
+#warning Not sure about gai_strerror, and this define check is stupid/incorrect too
+#warning but most likely it is missing too -> fake it
+
+#ifndef gai_strerror
+#define gai_strerror(x) "Generic DNS error"
+#endif
+
+#endif
+
+
+#if defined(__CYGWIN32__) || defined(__CYGWIN__)
+#include <cygwin/in.h>
+#endif
+
+#ifndef AI_NUMERICSERV
+#define AI_NUMERICSERV 0
+#endif
+
+#ifndef AI_ADDRCONFIG
+#define AI_ADDRCONFIG 0
+#endif
 
 class CAddrInfo {
 public:
