@@ -2,17 +2,31 @@
 #ifndef __CONFIG_H_
 #define __CONFIG_H_
 
+#include "../acsyscap.h"
+
+#define CREATPERMS 00664
+
 // keep this consistently, for experiments with other STL implementations
 #define MYSTD std
 //#define MYSTD ustl
 //#include <ustl.h>
 
-#ifndef USE_BOOST
+#ifdef HAVE_TR1_MEMORY
 #include <tr1/memory>
 #define SMARTPTR_SPACE MYSTD::tr1
-#else
+#elif defined HAVE_BOOST_SMARTPTR
 #include <boost/smart_ptr.hpp>
 #define SMARTPTR_SPACE boost
+#else
+#error Unable to find smart pointer implementation, install Boost or recent compiler and STL with TR1 features
+#endif
+
+#ifndef HAVE_ZLIB
+#warning Zlib or its development files are not available. Install them (e.g. zlib1g-dev) and run "make clean". Gzip format support disabled.
+#endif
+
+#ifndef HAVE_LIBBZ2
+#warning LibBz2 or its development files are not available. Install them (e.g. libbz2-dev) and run "make clean". Bzip2 format support disabled.
 #endif
 
 #define SHARED_PTR SMARTPTR_SPACE::shared_ptr
@@ -28,33 +42,10 @@
 #define MYCATCH catch
 #endif
 
-//#define atooff_t atoi
-#define tStrPos MYSTD::string::size_type 
-#define stmiss MYSTD::string::npos
-#define UINT unsigned int
-#define USHORT unsigned short
-
 #define __STDC_FORMAT_MACROS
 #include <inttypes.h>
 
-#ifdef WINDOWS
-#define WIN32
-#define SZPATHSEP "\\"
-#else
-#define SZPATHSEP "/"
-#endif
-
 #define ENEMIESOFDOSFS "?[]\\=+<>:;#"
 
-#ifndef _EMBED
-#define HAS_WORDEXP 1
-#define HAS_ADVISE 1
-#define HAS_IPV6SOCKETS 1
-
-#define HAS_ZLIB 1
-#define HAS_BZ2LIB 1
-
-#endif
-
 #endif // __CONFIG_H
 
