@@ -5,6 +5,7 @@
 #include <string>
 #include <list>
 #include <map>
+#include <set>
 
 #include <netinet/in.h>
 #include <netdb.h>
@@ -16,22 +17,14 @@
 
 class tDlJob;
 
-typedef enum {
-	STATE_GETHEADER
-	,STATE_GETDATA
-	,STATE_GETDATA_CHUNKED
-	,STATE_GETCHUNKHEAD
-	,STATE_FINISHJOB
-} tDlState;
 
 class dlcon : public lockable
 { 
-	
     public:
-        dlcon();
+        dlcon(bool bSingleRunMode, MYSTD::string *xff=NULL);
         ~dlcon();
                 
-        void WorkLoop(bool bSingleRun=false);
+        void WorkLoop();
         
         void SignalStop();
 
@@ -39,46 +32,44 @@ class dlcon : public lockable
         void AddJob(tFileItemPtr m_pItem, 
         		acfg::tHostiVec *pBackends, const MYSTD::string & sPatSuffix);
         //void Reset();
+
+        MYSTD::string m_sXForwardedFor;
         
     private:
     	
-    	inline void _AddJob(tDlJob*);
+    	friend class tDlJob;
+    	
+    	void _AddJob(tDlJob *);
 
     	MYSTD::list<tDlJob*> m_qToReceive;
     	int m_wakepipe[2];
     	
     	int m_conFd;
-    	MYSTD::string m_sConnectedHost; // used to remember to which host we are connected
+    	//MYSTD::string m_sConnectedHost; // used to remember to which host we are connected
 
-    	MYSTD::string m_sSendBuf, m_sErrorMsgBuf;
-    	MYSTD::string m_sReqBufForHost;
     	acbuf m_InBuf;
     	
     	// flags and local copies for input parsing
     	/// remember being attached to an fitem
-    	bool m_bIsResponsibleDler:1;
-    	bool m_bReconnectASAP:1;
-    	// released by the creator, allowed to terminate ASAP
-    	bool m_bOrphaned:1;
-    	
-    	off_t m_nRest;
-    	tDlState m_DlState;
+
+    	bool m_bSingleRun;
+    	    
     	tHttpUrl * m_proxy;
     
-    	void MakeRequest(tDlJob*);
-    	
-    	bool _CheckNoMoreUsers();
-    	bool _Connect(const MYSTD::string & sHostname);
-    	void _CleanupForFailedHost(const MYSTD::string & szErrorMsg);
-    	bool _HandleIncoming(MYSTD::string & sErrorOut);
-    	bool _HotDetach();
-    	bool _CleanOneJob();
+    	//MYSTD::set<tHttpUrl*> m_MirrorBlacklist;
+        MYSTD::set<MYSTD::string> m_MirrorHostBlacklist;
+        
+            	
+    	bool _Connect(const MYSTD::string & sHostname, MYSTD::string &sErrOut);
+    	inline void _Disconnect();
     	
     	//not to be copied
     	dlcon & operator=(const dlcon&);
     	dlcon(const dlcon&);
 };
 
+typedef MYSTD::list<tDlJob*>::iterator dljIter; 
+
 #endif
 
 
