@@ -5,10 +5,10 @@
 #include "config.h"
 #include "acbuf.h"
 
-#ifdef HAS_BZ2LIB
+#ifdef HAVE_LIBBZ2
 #include <bzlib.h>
 #endif
-#ifdef HAS_ZLIB
+#ifdef HAVE_ZLIB
 #include <zlib.h>
 #endif
 
@@ -39,10 +39,11 @@ public:
 	//! @return False on errors.
 	bool GetOneLine(MYSTD::string & sOut);
 	MYSTD::string GetPositionDescription();
-	void Close();
+	// void Close();
 	bool CheckGoodState(bool bTerminateOnErrors);
 	
-	static bool GetMd5Sum(const MYSTD::string & sFileName, uint8_t out[]);
+	static bool GetMd5Sum(const MYSTD::string & sFileName, uint8_t out[], bool bTryUnpack, off_t &scannedSize);
+	static bool GetSha1Sum(const MYSTD::string & sFileName, uint8_t out[], bool bTryUnpack, off_t &scannedSize);
 	static bool GetMd5String(const MYSTD::string & sFileName, char out[]);
 	static bool CheckMd5Sum(const MYSTD::string & sFileName, const MYSTD::string & sReference);
 	char *GetBuffer(size_t & size) { size=m_nBufSize; return m_szFileBuf;};
@@ -64,10 +65,10 @@ private:
 	//! Returns: currently usable remaining text buffer size
 	inline void _UncompressMoreData();
 	
-#ifdef HAS_BZ2LIB
+#ifdef HAVE_LIBBZ2
 	bz_stream m_bzStream;
 #endif
-#ifdef HAS_ZLIB
+#ifdef HAVE_ZLIB
 	z_stream m_zStream;
 #endif
 	
@@ -77,9 +78,5 @@ private:
 
 };
 
-MYSTD::string CsBinToString(const uint8_t b[]);
-bool CsAsciiToBin(const unsigned char a[], uint8_t b[]);
-bool CsEqual(const unsigned char a[], uint8_t b[]);
-
 #endif // __FILEREADER_H
 
