@@ -20,7 +20,7 @@ protected:
 	virtual void SendMsg(const char *data, size_t size);
 	bool _SendHelper(const char *data, size_t len, int flags);
 	virtual void EndTransfer();
-	MYSTD::string GetHostname();
+	MYSTD::string & GetHostname();
 	
 	void SendDecoration(bool bBegin, const char *httpcode="200", const char *szDecoFile=NULL);
 	void SetStyle(MYSTD::string&);
@@ -29,6 +29,7 @@ protected:
 private:
 	maintenance(const maintenance&);
 	maintenance& operator=(const maintenance&);
+	MYSTD::string m_sHostname;
 };
 
 
