@@ -9,11 +9,16 @@
 
 #include "acfg.h"
 #include "filereader.h"
+#include "rechecks.h"
 
-#ifdef HAS_WORDEXP
+#ifdef HAVE_WORDEXP
 #include <wordexp.h>
+#elif defined(HAVE_GLOB)
+#include <glob.h>
 #endif
 
+#include <errno.h>
+
 #include <iostream>
 #include <fstream>
 #include <string>
@@ -42,8 +47,11 @@ MapNameToString n2sTbl[] = {
 		,{ "ReportPage",&reportpage}
 		,{ "VfilePattern", &vfilepat}
 		,{ "PfilePattern", &pfilepat}
+		,{ "WfilePattern", &wfilepat}
 		,{ "AdminAuth",  &adminauth}
 		,{ "BindAddress", &bindaddr}
+		,{ "UserAgent", &agentname}
+		//,{ "Umask",		  &sUmask}
 };
 
 MapNameToInt n2iTbl[] = {
@@ -57,6 +65,10 @@ MapNameToInt n2iTbl[] = {
 		,{ "ExTreshold",	&extreshhold} 
 		,{ "MaxSpareThreadSets",	&tpoolsize}
 		,{ "DnsCacheSeconds", &dnscachetime}
+		,{ "UnbufferLogs", &debug }
+		,{ "ExAbortOnProblems", &exfailabort}
+		,{ "ExposeOrigin", &exporigin}
+		,{ "LogSubmittedOrigin", &logxff}
 };
 
 #define _iterPos(it, start) (it-start.begin())/sizeof(it)
@@ -121,9 +133,10 @@ bool _ReadMainConfiguration(const char * szFilename)
 		bNotEof=reader.GetOneLine(sLine);
 		if (IsValidButIrrelevantLine(sLine))
 			continue;
-		SetOption(sLine);
-		if (debug>4)
-			cout << key << " -> "<< val <<endl;
+		if(! SetOption(sLine))
+			BARF("Error reading main options, terminating.");
+		// XXX: move that to SetOption... if (debug>4)
+		//	cout << key << " -> "<< val <<endl;
 	}
 	return true;
 }
@@ -170,24 +183,37 @@ inline void _AddRemapInfo(bool bAsBackend, const string & token,
 		if (sPath[0]!=cPathSep)
 			sPath.insert(0, confdir+sPathSep);
 
-		// TODO: does it support glob? Use glob instead?
-#ifdef HAS_WORDEXP
+#ifdef HAVE_WORDEXP
 		wordexp_t p;
-		wordexp(sPath.c_str(), &p, 0);
-		for (UINT i=0; i<p.we_wordc; i++)
+		memset(&p, 0, sizeof(p));
+		if(0==wordexp(sPath.c_str(), &p, 0))
 		{
-			if (bAsBackend)
-				_ReadBackendsFiles(p.we_wordv[i], repname);
-			else
-				_ReadRewriteFiles(p.we_wordv[i], repname);
-		}
-		wordfree(&p);
+			for (UINT i=0; i<p.we_wordc; i++)
+			{
+				const string sToRead = p.we_wordv[i];
+#elif defined HAVE_GLOB
+		glob_t globbuf;
+		memset(&globbuf, 0, sizeof(glob_t));
+		if(0==glob(sPath.c_str(), GLOB_DOOFFS | GLOB_NOSORT, NULL, &globbuf))
+		{
+			for (unsigned int i=0; i<globbuf.gl_pathc; i++)
+			{
+				const string sToRead = globbuf.gl_pathv[i];
 #else
-		 
-		if (bAsBackend)
-			_ReadBackendsFiles(sPath, repname);
-		else
-			_ReadRewriteFiles(sPath, repname);
+				const string & sToRead = sPath;
+#endif		
+				if (bAsBackend)
+					_ReadBackendsFiles(sToRead, repname);
+				else
+					_ReadRewriteFiles(sToRead, repname);			
+#ifdef HAVE_WORDEXP
+			}
+			wordfree(&p);
+		}
+#elif defined HAVE_GLOB	
+			}
+			globfree(&globbuf);
+		}
 #endif
 	}
 }
@@ -210,12 +236,35 @@ bool SetOption(const MYSTD::string &sLine, bool bQuiet)
 	if(key.empty())
 		return false; // weird
 	
-	string * sTarget;
-	int * nTarget;
-	if ( NULL != (sTarget = _GetStringPtr(key)))
-		*sTarget=value;
-	else if ( NULL != (nTarget = _GetIntPtr(key)))
-		*nTarget=atoi(value.c_str());
+	string * psTarget;
+	int * pnTarget;
+	if ( NULL != (psTarget = _GetStringPtr(key)))
+		*psTarget=value;
+	else if ( NULL != (pnTarget = _GetIntPtr(key)))
+	{
+		const char *pStart=value.c_str();
+		if(! *pStart)
+		{
+			cerr << "Missing value for " << key << " option!" <<endl;
+			return false;
+		}
+		
+		errno=0;
+		char *pEnd;
+		*pnTarget=strtol(pStart, &pEnd, 10);
+		if (errno)
+		{
+			cerr << "Invalid number for " << key << " ";
+			perror("option");
+			return false;
+		}
+		if(*pEnd)
+		{
+			cerr << "Bad value for " << key << " option or found trailing garbage: " << pEnd <<endl;
+			return false;
+		}
+				
+	}
 	else if(0==strncasecmp(key.c_str(), "Remap-", 6))
 	{
 		string vname=key.substr(6, key.npos);
@@ -251,12 +300,12 @@ bool SetOption(const MYSTD::string &sLine, bool bQuiet)
 
 
 //const string * GetVnameForUrl(string path, string::size_type * nMatchLen)
-const string * GetRepNameAndPathResidual(const tHttpUrl in, MYSTD::string & sRetPathResidual)
+const string * GetRepNameAndPathResidual(const tHttpUrl & in, MYSTD::string & sRetPathResidual)
 {
 	sRetPathResidual.clear();
 	
 	pair<tUrl2RepIter,tUrl2RepIter> range=mapUrl2pVname.equal_range(in);
-	if(range.first==mapUrl2pVname.end())
+	if(range.first==range.second)
 		return NULL;
 	
 	tStrPos bestMatchLen(0);
@@ -344,7 +393,8 @@ void _ReadBackendsFiles(const string & sFile, const string &sRepName)
 				}
 			}
 			*/
-			else if(keyEq("Archive-http", key) || keyEq("X-Archive-Http", key))
+			else if(keyEq("Archive-http", key) || keyEq("X-Archive-http", key) 
+					|| keyEq("Volatile-http",key) || keyEq("X-Volatile-http", key))
 			{
 				_FixPostPreSlashes(val);
 				entry.sPath=val;
@@ -381,6 +431,7 @@ void _ReadRewriteFiles(const string & sFile, const string & sRepName)
 	for(bool bNotEof=true; bNotEof; )
 	{
 		bNotEof=reader.GetOneLine(sLine);
+
 		if (0==sLine.compare(0, 7, "http://"))
 		{ // TODO: The check above is optional since SetHttpUrl does that too, but it's 
 			// more userfriendly on errors (more exact error message)
@@ -425,11 +476,11 @@ void _ReadRewriteFiles(const string & sFile, const string & sRepName)
 					url.sPath=*itPath;
 					pair<tHttpUrl,const string*> info(url, _CheckBEentryGetNamePtr(sRepName));
 					mapUrl2pVname.insert(info);
-/*
-					if (debug)
-						cout << "Mapping: "<< *itHost << ""<< *itPath 
-						<< " -> "<< itHostiVec->first<<endl;
-*/
+
+#ifdef DEBUG
+						cout << "Mapping: "<< *itHost << *itPath 
+						<< " -> "<< sRepName <<endl;
+#endif
 				}
 			}
 			hosts.clear();
@@ -443,10 +494,11 @@ void _ReadRewriteFiles(const string & sFile, const string & sRepName)
 		}
 		
 		// got something, intepret it...
-		
 		if( keyEq("Site", key) || keyEq("Alias", key) || keyEq("Aliases", key))
-			Tokenize(val, SPACECHARS, hosts);
-		if(keyEq("Archive-http", key) || keyEq("X-Archive-http", key))
+			Tokenize(val, SPACECHARS, hosts, true);
+		
+		if(keyEq("Archive-http", key) || keyEq("X-Archive-http", key) 
+				|| keyEq("Volatile-http",key) || keyEq("X-Volatile-http", key))
 		{
 			// help STL saving some memory
 			if(sPopularPath==val)
@@ -469,7 +521,7 @@ void ReadConfigDirectory(const char *szPath)
 	realpath(szPath, buf);
 	confdir=buf; // pickup the last config directory
 
-#ifdef HAS_WORDEXP
+#ifdef HAVE_WORDEXP
 	wordexp_t p;
     wordexp((confdir+"/*.conf").c_str(), &p, 0);
     for (UINT i=0; i<p.we_wordc; i++)
@@ -536,10 +588,43 @@ void PostProcConfig()
 	
     // postprocessing
 
+#if 0
+	if(!sUmask.empty())
+	{
+		mode_t nUmask=0;
+		if(sUmask.size()>4)
+		{
+			cerr << "Invalid umask length\n";
+			exit(EXIT_FAILURE);
+		}
+		for(unsigned int i=0; i<sUmask.size(); i++)
+		{
+			unsigned int val = sUmask[sUmask.size()-i-1]-'0';
+			if(val>7)
+			{
+				cerr << "Invalid umask value\n" <<endl;
+				exit(EXIT_FAILURE);
+			}
+			nUmask |= (val<<(3*i));
+		
+		}
+		//cerr << "Got umask: " << nUmask <<endl;
+		umask(nUmask);
+	}
+#endif
+	
    if(cachedir.empty() || cachedir[0] != sPathSep[0]) {
       cerr << "Cache directory unknown or not absolute, terminating..." <<endl;
       exit(EXIT_FAILURE);
    }
+   
+   rechecksEx rex;
+   if(rex.IsRegexOk())
+   {
+	   cerr << "An error occured while compiling file type regular expression!" <<endl;
+	   exit(EXIT_FAILURE);
+   }
+   
    // get rid of duplicated and trailing slash(es)
    tStrVec tmp;
    Tokenize(cachedir, sPathSep.c_str(), tmp);
@@ -564,6 +649,9 @@ void PostProcConfig()
 		   proxy_info.sPath=string("Proxy-Authorization: Basic ")+_GetBase64Auth(proxy.substr(0, pos))+"\r\n";
 		   proxy_info.sHost.erase(0,pos+1);
 	   }
+     else
+        proxy_info.sPath.clear(); // no auth at all, stop confusion
+
 	   if(proxy_info.sHost.empty())
 	   {
 		   cerr << "Invalid proxy specification, aborting..." << endl;
@@ -611,7 +699,7 @@ void PostProcConfig()
 			cout << n2iTbl[i].name << " = \"" << *(n2iTbl[i].ptr) << "\"\n";
 	}
 #else
-   if(acfg::debug>0)
+   if(acfg::debug>2)
 	   cout << "\n\nAdditional debugging information not compiled in.\n\n";
 #endif
    
