@@ -1,4 +1,9 @@
 
+// make sure the right function strerror_r is used
+#define _XOPEN_SOURCE 600
+#include <string.h>
+#include <errno.h>
+
 #include "debug.h"
 
 #include "aclogger.h"
@@ -48,7 +53,19 @@ void transfer(bool bDirIn, uint64_t nCount, const char *szClient, const char *sz
 			(unsigned long) time(NULL), bDirIn ? 'I' : 'O', nCount,
 					szClient, szPath);
 
-	if(acfg::debug>3)
+	if(acfg::debug)
+		fflush(fStat);
+}
+
+void misc(const string & sLine)
+{
+	if(!fStat)
+		return;
+	lockguard g(&mx);
+
+	fprintf(fStat, "%lu|M|%s\n", (unsigned long) time(NULL), sLine.c_str());
+	
+	if(acfg::debug)
 		fflush(fStat);
 }
 
@@ -164,7 +181,9 @@ void GetStats(vector<tRowData> & out)
 				 if(when>now || delta>=3600*24*28)
 					 continue;
 				 
-#define RANGEMARK 1234
+				 // uber-maximum of days we would ever display, use as offset to mark rows for weekly style presentation 
+#define RANGEMARK 2048
+				 
 				 time_t key; // map to particular weeks or DOW
 				 if(delta>=7*24*3600) // account for weeks
 					key = RANGEMARK + delta / (7*3600*24); 
@@ -176,13 +195,13 @@ void GetStats(vector<tRowData> & out)
 				 switch(* tokens[1].c_str()) {
 				 case('I'): mapPeriod2InOut[key].first+=count; break;
 				 case('O'): mapPeriod2InOut[key].second+=count; break;
-				 default: return; // what the...
+				 default: continue;
 				 }
 			}
 		}
 		globfree(&globbuf);
 		
-		// Gathering...
+		// Converting stats to (almost) user readable output for caller 
 		
 		map<time_t,pair<uint64_t,uint64_t> >::iterator it;
 		for(it=mapPeriod2InOut.begin(); it!=mapPeriod2InOut.end(); it++)
@@ -198,7 +217,7 @@ void GetStats(vector<tRowData> & out)
 				data.from=data.to-7*24*3600;
 			}
 			out.push_back(data);
-			// TODO: serialize, into the dat cache file
+			
 		}
 
 	}
@@ -226,5 +245,19 @@ void GetStats(vector<tRowData> & out)
 	}
 }
 	
+errnoFmter::errnoFmter()
+{
+	char buf[32];
+	buf[0]=buf[31]=0x0;
+	msg=&buf[0];
+
+#ifdef _GNU_SOURCE
+#warning COMPILER BUG DETECTED -- _GNU_SOURCE is set in C++ mode
+	msg=strerror_r(errno, buf, sizeof(buf)-1);
+#else
+	strerror_r(errno, buf, sizeof(buf)-1);
+#endif
+}
+
 }
 
