@@ -20,9 +20,12 @@ static void usage();
 static void _TestCacheDir();
 void term_handler(int signum);
 void log_handler(int signum);
+void check_algos();
 
 int main(int argc, char **argv) {
 
+	check_algos();
+	
 	signal(SIGTERM, term_handler);
 	signal(SIGINT, term_handler);
 	signal(SIGQUIT, term_handler);
@@ -172,7 +175,7 @@ void term_handler(int signum)
 #ifdef PLAYGROUND
 
 /* 
- * Let's be another md5sum tool...
+ * Let's be another csum tool...
  
 	md5_state_s ctx;
 	md5_init(&ctx);
@@ -182,21 +185,21 @@ void term_handler(int signum)
 		int n=fread(buf, sizeof(char), 2000, stdin);
 		md5_append(&ctx, buf, n);
 	}
-	uint8_t sum[16];
-	md5_finish(&ctx, sum);
+	uint8_t csum[16];
+	md5_finish(&ctx, csum);
 	for(int i=0;i<16;i++)
-		printf("%02x", sum[i]);
+		printf("%02x", csum[i]);
 	printf("\n");
 	exit(0);
 */
 	/*
-	uint8_t sum[16];
+	uint8_t csum[16];
 	string s(argv[1]);
-	filereader::GetMd5Sum(s, sum);
+	filereader::GetMd5Sum(s, csum);
 	for(int i=0;i<16;i++)
-			printf("%02x", sum[i]);
+			printf("%02x", csum[i]);
 		printf("\n");
-		if(CsEqual((unsigned char*) argv[2], sum))
+		if(CsEqual((unsigned char*) argv[2], csum))
 			printf("Is OK\n");
 		else
 			printf("Differenz\n");
