@@ -42,7 +42,7 @@ bool CAddrInfo::Resolve(const string & sHostname, const string &sPort,
 
 	if (0!=r)
 	{
-		sErrorBuf="503 ";
+		sErrorBuf="503 DNS error: ";
 		sErrorBuf+=gai_strerror(r);
 		return false;
 	}
