@@ -2,7 +2,8 @@
 #include "conserver.h"
 #include "meta.h"
 #include "lockable.h"
-#include "con.h"
+#include "conn.h"
+#include "rfc2553emu.h"
 #include "acfg.h"
 
 
@@ -16,6 +17,7 @@
 
 #include <netdb.h>
 
+#include <cstdio>
 #include <list>
 #include <map>
 #include <iostream>
@@ -24,6 +26,11 @@
 
 using namespace MYSTD;
 
+// for cygwin, incomplete ipv6 stuff
+#ifndef AF_INET6
+#define AF_INET6        23              /* IP version 6 */
+#endif
+
 namespace conserver
 {
 
@@ -226,12 +233,6 @@ void Setup()
 			trimString(*it);
 		    struct addrinfo *res, *p;
 		    
-	    	// kludge, to be FIXED!
-		    // avoid bitching when V6 socket was used since v6/v4 bridge should be in place already.
-	    	// TODO: Decide, how to handle other funny kinds of collisions?
-	    	// Can we rely on any specific order from getaddrinfo?
-	    	bool bV6used=false;
-		    
 		    if(0!=getaddrinfo(bindaddr.empty() ? NULL : it->c_str(),
 				   port.c_str(), &hints, &res))
 			   goto error_getaddrinfo;
@@ -241,34 +242,46 @@ void Setup()
 		    	int sockip = socket(p->ai_family, p->ai_socktype, p->ai_protocol);
 				if (sockip<0)
 					goto error_socket;
-				
+
+
+#ifdef IPV6_V6ONLY
+		    	if(p->ai_family==AF_INET6)
+		    		setsockopt(sockip, SOL_IPV6, IPV6_V6ONLY, &yes, sizeof(yes));
+#endif		    	
 				setsockopt(sockip, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(yes));
 				
+				
 		    	if (bind(sockip, p->ai_addr, p->ai_addrlen))
 		    		goto error_bind;
 		    	if (listen(sockip, SO_MAXCONN))
 		    		goto error_listen;
 		    	
 		    	vecSocks.push_back(sockip);
-		    
-		    	if(p->ai_family==AF_INET6)
-		    		bV6used=true;
 		    	
 		    	continue;
-		    	
-		    	error_socket:
-		    	if(yes) // reuse var, does not matter anymore
-		    		perror("Error creating socket");
-		    	yes=0;
-		    	error_listen:
-		    	if(yes)
-		    		perror("Couldn't listen on socket");
-		    	yes=0;
-		    	error_bind:
-		    	if(yes && !bV6used)
-		    		perror("Couldn't bind socket");
-		    	close(sockip);
-		    	sockip=-1;
+
+				error_socket:
+				
+				if(EAFNOSUPPORT != errno &&
+						EPFNOSUPPORT != errno &&
+						ESOCKTNOSUPPORT != errno &&
+						EPROTONOSUPPORT != errno)
+				{
+					perror("Error creating socket");
+				}
+				goto close_socket;
+
+				error_listen:
+				perror("Couldn't listen on socket");
+				goto close_socket;
+
+				error_bind:
+				perror("Couldn't bind socket");
+				goto close_socket;
+
+				close_socket:
+				close(sockip);
+				sockip=-1;
 		    }
 		    freeaddrinfo(res);
 		    continue;
