@@ -7,10 +7,12 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <sys/mman.h>
+#include <errno.h>
 
-#include <sys/mman.h>
+#include <limits>
 
-#include "../3rd-party/md5.h"
+#include "md5.h"
+#include "sha1.h"
 
 
 #define FLAG_PLAIN 1
@@ -56,13 +58,12 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 	
 	if(m_fd<0)
 		goto ofail;
-	
+
 	if(bNoMagic)
 		flags|=FLAG_PLAIN;
-	else if(sFilename.length()>4 
-			&& 0==strcasecmp(".bz2", sFilename.c_str()+sFilename.length()-4))
+	else if(endsWithSzAr(sFilename, ".bz2"))
 	{
-#ifdef HAS_BZ2LIB
+#ifdef HAVE_LIBBZ2
 		m_bzStream.bzalloc = NULL;
 		m_bzStream.bzfree = NULL;
 		m_bzStream.opaque = NULL;
@@ -76,10 +77,9 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 		goto ofail;
 #endif
 	}
-	else if(sFilename.length()>3 
-			&& 0==strcasecmp(".gz", sFilename.c_str()+sFilename.length()-3))
+	else if(endsWithSzAr(sFilename, ".gz"))
 	{
-#ifdef HAS_ZLIB
+#ifdef HAVE_ZLIB
 		m_zStream.zalloc=NULL;
 		m_zStream.zfree=NULL;
 		m_zStream.opaque=NULL;
@@ -100,6 +100,13 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 	struct stat statbuf;
 	if(0!=fstat(m_fd, &statbuf))
 		goto ofail;
+
+	// LFS on 32bit? That's not good for mmap. Don't risk incorrect behaviour.
+	if(uint64_t(statbuf.st_size) >  std::numeric_limits<size_t>::max())
+    {
+        errno=EFBIG;
+		goto ofail; 
+    }
 	
 	m_nBufSize=statbuf.st_size;
 	m_szFileBuf = (char*) mmap(0, statbuf.st_size, PROT_READ, MAP_SHARED, m_fd, 0);
@@ -107,7 +114,7 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 	if(m_szFileBuf==MAP_FAILED)
 		goto ofail;
 	
-#ifdef HAS_ADVISE
+#ifdef HAVE_MADVISE
 	// if possible, prepare to read that
 	posix_madvise(m_szFileBuf, statbuf.st_size, POSIX_MADV_SEQUENTIAL);
 #endif
@@ -150,12 +157,12 @@ filereader::~filereader() {
 	if(m_fd>=0)
 		close(m_fd);
 	
-#ifdef HAS_BZ2LIB
+#ifdef HAVE_LIBBZ2
 	if(flags&FLAG_BZ2)
 		BZ2_bzDecompressEnd (&m_bzStream);
 #endif
 
-#ifdef HAS_ZLIB
+#ifdef HAVE_ZLIB
 	if(flags&FLAG_GZ)
 		deflateEnd(&m_zStream);
 #endif
@@ -248,16 +255,16 @@ inline void filereader::_UncompressMoreData() {
 		|| m_UncompBuf.freecapa()==0 )
 		return;
 	
-	if(m_nBufPos>m_nBufSize )
+	if(m_nBufPos>=m_nBufSize )
 	{
-		// shouldn't be here. Decompressor not finished? Unexpected EOF!
+		// shouldn't be here. Decompressor's errors or eof must have been handled before. Undiscovered somehow?
 		flags|=(FLAG_ERROR|FLAG_EOF);
 		return;
 	}
 	
 	unsigned int nFeedLen=m_nBufSize-m_nBufPos;
 	
-#ifdef HAS_BZ2LIB
+#ifdef HAVE_LIBBZ2
 	if(flags&FLAG_BZ2)
 	{
 		m_bzStream.next_in=m_szFileBuf+m_nBufPos;
@@ -282,7 +289,7 @@ inline void filereader::_UncompressMoreData() {
 		else flags|=(FLAG_COMPEOF|FLAG_ERROR);
 	}
 #endif
-#ifdef HAS_ZLIB
+#ifdef HAVE_ZLIB
 	if(flags&FLAG_GZ)
 	{
 		m_zStream.next_in=(Bytef*) m_szFileBuf+m_nBufPos;
@@ -308,7 +315,8 @@ inline void filereader::_UncompressMoreData() {
 #endif
 }
 
-// TODO: make this use a string
+/*
+// TODO: make this use a string?
 bool filereader::GetMd5String(const MYSTD::string & sFileName, char out[])
 {
 	uint8_t buf[16];
@@ -320,17 +328,21 @@ bool filereader::GetMd5String(const MYSTD::string & sFileName, char out[])
 	
 	return true;
 }
+*/
 
-
-bool filereader::GetMd5Sum(const MYSTD::string & sFileName, uint8_t out[])
+bool filereader::GetMd5Sum(const MYSTD::string & sFileName, uint8_t out[], bool bTryUnpack, off_t &scannedSize)
 {
 	md5_state_s ctx;
 	md5_init(&ctx);
 	filereader f;
-	if (!f.OpenFile(sFileName, true))
+	scannedSize=0;
+	if (!f.OpenFile(sFileName, !bTryUnpack))
 		return false;
 	if (f.flags&FLAG_PLAIN)
+	{
 		md5_append(&ctx, (md5_byte_t*) f.m_szFileBuf, f.m_nBufSize);
+		scannedSize=f.m_nBufSize;
+	}
 	else
 	{
 		while(true)
@@ -342,14 +354,81 @@ bool filereader::GetMd5Sum(const MYSTD::string & sFileName, uint8_t out[])
 			if(nRest==0)
 				break;
 			md5_append(&ctx, (md5_byte_t*) f.m_UncompBuf.rptr(), nRest);
+			scannedSize+=nRest;
 			f.m_UncompBuf.clear();
 		}
 		
 	}
+	
 	md5_finish(&ctx, out);
-	return true;
+	return f.CheckGoodState(false);
 }
 
+bool filereader::GetSha1Sum(const MYSTD::string & sFileName, uint8_t out[], bool bTryUnpack, off_t &scannedSize)
+{
+	SHA_INFO ctx;
+	sha_init(&ctx);
+	filereader f;
+	scannedSize=0;
+	if (!f.OpenFile(sFileName, !bTryUnpack))
+		return false;
+	if (f.flags&FLAG_PLAIN)
+	{
+		sha_update(&ctx, (SHA_BYTE*) f.m_szFileBuf, f.m_nBufSize);
+		scannedSize=f.m_nBufSize;
+	}
+	else
+	{
+		while (true)
+		{
+			f._UncompressMoreData();
+			if (f.flags&FLAG_ERROR)
+				return false;
+			UINT nRest=f.m_UncompBuf.size();
+			if (nRest==0)
+				break;
+			sha_update(&ctx, (md5_byte_t*) f.m_UncompBuf.rptr(), nRest);
+			scannedSize+=nRest;
+			f.m_UncompBuf.clear();
+		}
+
+	}
+	sha_final(out, &ctx);
+	return f.CheckGoodState(false);
+}
+
+void check_algos()
+{
+	const char testvec[]="abc";
+	uint8_t out[20];
+	 
+	{
+		SHA_INFO ctx;
+		sha_init(&ctx);
+		sha_update(&ctx, (SHA_BYTE*) testvec, sizeof(testvec)-1);
+		sha_final(out, &ctx);
+		if (CsBinToString(out, 20)!="a9993e364706816aba3e25717850c26c9cd0d89d")
+		{
+			cerr
+					<< "Incorrect SHA1 implementation detected, check compilation settings!\n";
+			exit(EXIT_FAILURE);
+		}
+	}
+	{
+		md5_state_s ctx;
+		md5_init(&ctx);
+		md5_append(&ctx, (md5_byte_t*) testvec, sizeof(testvec)-1);
+		md5_finish(&ctx, out);
+		if (CsBinToString(out, 16)!="900150983cd24fb0d6963f7d28e17f72")
+		{
+			cerr
+					<< "Incorrect MD5 implementation detected, check compilation settings!\n";
+			exit(EXIT_FAILURE);
+		}
+	}
+}
+
+
 /*
 foreach $b (0..255) {
    print "\n" if($b%16==0);
@@ -382,9 +461,9 @@ uint_fast16_t hexmap[] = {
                 _inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv
                 };
 
-inline bool CsEqual(const unsigned char a[], uint8_t b[])
+inline bool CsEqual(const unsigned char a[], uint8_t b[], unsigned short binLen)
 {
-	for(int i=0; i<16;i++)
+	for(int i=0; i<binLen;i++)
 	{
 		uint_fast16_t r=hexmap[a[i*2]] * 16 + hexmap[a[i*2+1]];
 		if(r != b[i]) return false;
@@ -392,31 +471,34 @@ inline bool CsEqual(const unsigned char a[], uint8_t b[])
 	return true;
 }
 
-bool CsAsciiToBin(const unsigned char a[], uint8_t b[])
+bool CsAsciiToBin(const char *a, uint8_t b[], unsigned short binLength)
 {
-	for(int i=0; i<16;i++)
+	const unsigned char *uA = (const unsigned char*) a;
+	for(int i=0; i<binLength;i++)
 	{
-		uint_fast16_t r=hexmap[a[i*2]] * 16 + hexmap[a[i*2+1]];
+		uint_fast16_t r=hexmap[uA[i*2]] * 16 + hexmap[uA[i*2+1]];
 		if(r>255) return false;
 		b[i]=r;
 	}
 	return true;
 }
 
-string CsBinToString(const uint8_t sum[])
+string CsBinToString(const uint8_t sum[], unsigned short lengthBin)
 {
-	char szRet[32];
-	
-	char hexmap[] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
-	for(int i=0; i<16; i++)
+	string sRet;
+	char hexmap[] =
+	{ '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd',
+			'e', 'f' };
+	for (int i=0; i<lengthBin; i++)
 	{
-		szRet[2*i]=hexmap[sum[i]>>4];
-		szRet[2*i+1]=hexmap[sum[i]&0xf];
+		sRet+=(char) hexmap[sum[i]>>4];
+		sRet+=(char) hexmap[sum[i]&0xf];
 	}
-		
-	return string(szRet, 32);
+
+	return sRet;
 }
 
+/*
 bool filereader::CheckMd5Sum(const string & sFileName, const string & sReference)
 {
 	uint8_t sum[16];
@@ -427,93 +509,7 @@ bool filereader::CheckMd5Sum(const string & sFileName, const string & sReference
 	if(!GetMd5Sum(sFileName, sum))
 		return false;
 	
-	return CsEqual( (unsigned char*) sReference.data(), sum);
+	return CsEqual( (unsigned char*) sReference.data(), sum, 16);
 }
 
-	/*
-
-#warning bla, weg mit
-	return true;
-	
-	if(flags&FLAG_EOF)
-		return false; // cannot get more
-	
-	// where to read from file?
-	acbuf readbuf & = (flags&FLAG_PLAIN) ? m_textbuf : m_compbuf;
-	if(m_fd>=0 && readbuf.
-	{
-		
-		
-	}
-    if(m_fd<0 && m_compbuf.empty())
-    {
-    	flags|=FLAG_ERROR;
-    	return false;
-    }
-    if( 0==(flags&FLAG_PLAIN))
-    dbglvl(DBGSPAM);
-    total_read+=nInBytes;
-    rem-=nInBytes;
-    //cerr << "total_read: " << total_read<<endl;
-
-    if(dtype==GZ) {
-        zstrm->next_in=(Bytef*) prebuf->rptr();
-        zstrm->avail_in=prebuf->size();
-        zstrm->next_out=(Bytef*) buf->wptr();
-        zstrm->avail_out=buf->freecapa();
-        dbglvl(DBGSPAM);
-
-        int ret=inflate(zstrm, Z_NO_FLUSH);
-        if(ret==Z_OK || ret==Z_STREAM_END) { //ok, accept the data
-            prebuf->drop(prebuf->size() - zstrm->avail_in);
-            nInBytes=buf->freecapa() - zstrm->avail_out; // now only interesting for real data
-            buf->got(nInBytes);
-            buf->moveData();
-        }
-        else { // corrupted data, or buffer overfull because of invalid stuff
-           erReturn;
-        }
-        if(ret==Z_STREAM_END)
-            eof=true;
-        dbglvl(DBGSPAM);
-    }
-    else if(dtype==BZ) {
-        dbglvl(DBGSPAM);
-        bzstrm->next_in=prebuf->rptr();
-        bzstrm->avail_in=prebuf->size();
-        bzstrm->next_out=buf->wptr();
-        bzstrm->avail_out=buf->freecapa();
-
-        int ret=BZ2_bzDecompress(bzstrm);
-        if(ret==BZ_OK || ret==BZ_STREAM_END) { //ok, take the data
-            dbglvl(DBGSPAM);
-            prebuf->drop(prebuf->size() - bzstrm->avail_in);
-            nInBytes=buf->freecapa() - bzstrm->avail_out;
-            buf->got(nInBytes);
-            buf->moveData();
-        }
-        else { // corrupted data, or buffer overfull because of invalid stuff
-            dbglvl(DBGSPAM);
-            erReturn;
-        }
-        if(ret==BZ_STREAM_END)
-            eof=true;
-        dbglvl(DBGSPAM);
-    }
-    else if(nInBytes==0 && rem != 0) // incomplete file?
-        eof=true;
-
-    dbglvl(DBGSPAM);
-    if(nInBytes)
-        process();
-
-    dbglvl(DBGSPAM);
-    buf->moveData();
-
-    if(eof) {
-        finish();
-        return 0;
-    }
-    // assume there is data, can be checked in the next run
-    return 1;
-    */
+*/
