@@ -8,6 +8,7 @@
 #include "config.h"
 #include <acbuf.h>
 
+#include <cstdio>
 #include <time.h>
 #include <iostream>
 #include <string.h>
@@ -29,6 +30,7 @@ static const char *szHeadNames[] = {
     	  "Transfer-Encoding", // 8
     	  "X-Original-Source",
     	  "Authorization",
+    	  "X-Forwarded-For",
     	  "X-42-Unreachable"
 };
 
@@ -222,7 +224,7 @@ int header::StoreToFile(const string &sPath)
 {
 	int nByteCount(0);
 	
-	int fd=open(sPath.c_str(), O_WRONLY|O_CREAT|O_TRUNC, 00644);
+	int fd=open(sPath.c_str(), O_WRONLY|O_CREAT|O_TRUNC, CREATPERMS);
 	if(fd<0)
 		return -errno;
 	
