@@ -3,17 +3,21 @@
 #include "debug.h"
 
 #include "job.h"
+#include <cstdio>
 #include <stdexcept>
 #include <limits>
 using namespace MYSTD;
 
-#include "con.h"
+#include "conn.h"
 #include "acfg.h"
 #include "fileitem.h"
 #include "dlcon.h"
 #include "maintenance.h"
 
+#ifdef HAVE_SENDFILE
 #include <sys/sendfile.h>
+#endif
+
 #include <errno.h>
 
 // optimization on Linux; no-op for platforms that don't know it
@@ -36,6 +40,8 @@ ssize_t sendfile_generic(int out_fd, int in_fd, off_t *offset, size_t count)
 		errno=EFAULT;
 		return -1;
 	}
+	if(lseek(in_fd, *offset, SEEK_SET)== (off_t)-1)
+		return -1;
 	while(count>0)
 	{
 		int readcount=read(in_fd, buf, count>sizeof(buf)?sizeof(buf):count);
@@ -69,6 +75,7 @@ ssize_t sendfile_generic(int out_fd, int in_fd, off_t *offset, size_t count)
 	return cnt;
 }
 
+#ifdef HAVE_SENDFILE
 ssize_t linux_sendfile_with_fallback(int out_fd, int in_fd, off_t *offset, size_t count)
 {
 	ssize_t r=sendfile(out_fd, in_fd, offset, count);
@@ -78,7 +85,9 @@ ssize_t linux_sendfile_with_fallback(int out_fd, int in_fd, off_t *offset, size_
 	else
 		return r;
 }
-
+#else
+#define linux_sendfile_with_fallback(out_fd, in_fd, poffset, count) sendfile_generic(out_fd, in_fd, poffset, count)
+#endif
 
 #ifndef __linux__
 #define sendfile_linux(a,b,c,d) sendfile_generic(a,b,c,d)
@@ -105,12 +114,17 @@ job::job(header *h, con *pParent) :
 
 job::~job()
 {
+	m_pParentCon->LogDataCounts(m_sFileLoc,
+			m_pReqHead->h[header::XFORWARDEDFOR],
+			(m_pItem ? m_pItem->GetTransferCount() : 0),
+			m_nAllDataCount);
+	
 	if (m_filefd>=0)
 		close(m_filefd);
 
 	if (m_pItem)
 	{
-		DelUser(m_pItem->m_sKey);
+		fileitem::DelUser(m_pItem->m_sKey);
 		m_pItem.reset();
 	}
 	
@@ -123,15 +137,6 @@ job::~job()
 	ldbg("job destroyed");
 }
 
-
-const MYSTD::string & job::TakeCounts(off_t &nSizeIn, off_t &nSizeOut)
-{
-	nSizeIn=m_pItem ? m_pItem->GetTransferCount() : 0;
-	nSizeOut=m_nAllDataCount;
-	m_nAllDataCount=0;
-	return m_sFileLoc;
-}
-
 void replaceChars(string &s, const char *szBadChars, char goodChar)
 {
 	for(string::iterator p=s.begin();p!=s.end();p++)
@@ -248,7 +253,7 @@ void job::PrepareDownload() {
     }
     
     bForceFreshnessChecks = ( ! acfg::offlinemode && m_type==rechecks::FILE_INDEX);
-    m_pItem=GetFileItem(m_sFileLoc);
+    m_pItem=fileitem::GetFileItem(m_sFileLoc);
     
     if(!m_pItem)
     {
@@ -270,7 +275,7 @@ void job::PrepareDownload() {
     if( fistate < FIST_DLGOTHEAD) // needs a downloader 
     {
     	dbgline;
-    	if(!m_pParentCon->SetupDownloader())
+    	if(!m_pParentCon->SetupDownloader(m_pReqHead->h[header::XFORWARDEDFOR]))
     	{
     		if(acfg::debug)
     			aclog::err(string("Error creating download handler for ")+m_sFileLoc);
@@ -284,15 +289,19 @@ void job::PrepareDownload() {
 			if(psVname && NULL != (pBackends=acfg::GetBackendVec(psVname)))
 			{
 				ldbg("Backends found, using them with " + sTmp);
+				ldbg("Passing new job to " << m_pParentCon->m_dlerthr);
 				m_pParentCon->m_pDlClient->AddJob(m_pItem, pBackends, sTmp);
+				
 			}
 			else
 			{
 			    if(acfg::forcemanaged)
 			    	goto report_notallowed;
 			    
+			    ldbg("Passing new job for " << tUrl.ToString() << " to " << m_pParentCon->m_dlerthr);
 			    m_pParentCon->m_pDlClient->AddJob(m_pItem, tUrl);
 			}
+			ldbg("Download job enqueued for " << m_sFileLoc);
 		}
 		MYCATCH(std::bad_alloc) // OOM, may this ever happen here?
 		{
@@ -330,7 +339,7 @@ report_invport:
 }
 
 #define THROW_ERROR(x) { m_sErrorMsgBuffer=x; goto THROWN_ERROR; }
-eJobResult job::SendData(int confd)
+job::eJobResult job::SendData(int confd)
 {
 	if(!m_sMaintCmd.empty())
 	{
@@ -360,27 +369,30 @@ eJobResult job::SendData(int confd)
 		
 		while (true)
 		{
-			fistate=m_pItem->status;
+			fistate=m_pItem->GetStatusUnlocked();
 			nNewSize=m_pItem->m_nSizeChecked;
 			
+			dbgline;
 			if (fistate>=FIST_ERROR)
 			{
-				m_sErrorMsgBuffer=m_pItem->GetHeader()->frontLine.c_str()+9;
-				if(m_sErrorMsgBuffer.empty())
-					m_sErrorMsgBuffer="500 Unknown error";
-				// to send the error response
-				return R_AGAIN;
+				const string & sErLine = m_pItem->GetHeaderUnlocked()->frontLine;
+				m_sErrorMsgBuffer = (sErLine.length()<10) ? "500 Unknown error" : sErLine.substr(9);
+				return R_AGAIN; // to send the error response
 			}
 			
+			dbgline;
 			if(fistate==FIST_COMPLETE || (m_nSendPos < nNewSize && fistate>=FIST_DLGOTHEAD))
 				break;
 			
+			dbgline;
 			m_pItem->wait();
+			
+			dbgline;
 		}
 		
 		// left loop with usefull state of fileitem, get its header once
 		if(m_RespHead.type == header::INVALID)
-			m_RespHead = *(m_pItem->GetHeader());
+			m_RespHead = *(m_pItem->GetHeaderUnlocked());
 
 	}
 	else if(m_state != STATE_SEND_BUFFER)
@@ -702,19 +714,25 @@ eJobResult job::SendData(int confd)
 			
 	THROWN_ERROR:
 			ldbg("Processing error: " << m_sErrorMsgBuffer);
-			if(m_nAllDataCount==0) // don't send errors when data header is underway
+			if(0==m_nAllDataCount) // don't send errors when data header is underway
 			{
 				m_sPrependBuf=string("HTTP/1.1 ")+m_sErrorMsgBuffer
 				+ "\r\nConnection: " + (m_bCloseCon?"close\r\n":"Keep-Alive\r\n")
-				+ header::GetInfoHeaders()+"X-Original-Source: " /*;
-				
+				+ header::GetInfoHeaders();
+								
 				if(m_RespHead.h[header::XORIG])
+				{
+					m_sPrependBuf+="X-Original-Source: ";
 					m_sPrependBuf+=m_RespHead.h[header::XORIG];
-				else
-					m_sPrependBuf+=m_sFileLoc;
-				m_sPrependBuf+= "\r\n\r\n";*/
-				+ (m_RespHead.h[header::XORIG]?m_RespHead.h[header::XORIG]:m_sFileLoc)
-				+ "\r\n\r\n";
+					m_sPrependBuf+="\r\n";
+				}
+				else if(!m_sFileLoc.empty())
+				{
+					m_sPrependBuf+="X-Original-Source: ";
+					m_sPrependBuf+=m_sFileLoc+"\r\n";
+				}
+				
+				m_sPrependBuf+="\r\n";
 				m_backstate=STATE_ERRORCONT;
 				m_sErrorMsgBuffer.clear();
 				USRDBG(REPLEVEL, "Response error or message, will continue (for " << m_sFileLoc<<")");
