@@ -6,6 +6,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <cstdio>
 #include <iostream>
 using namespace MYSTD;
 
@@ -106,23 +107,40 @@ void find_base_name(const char *in, const char * &pos, UINT &len)
 }
 */
 
-tStrVec::size_type Tokenize(const string & in, const char *sep, tStrVec & out)
+tStrVec::size_type Tokenize(const string & in, const char *sep, tStrVec & out, bool bAppend)
 {
+	if(!bAppend)
+		out.clear();
+	tStrVec::size_type nBefore(out.size());
+	
 	tStrPos pos=0, pos2=0, oob=in.length();
 	while (pos<oob)
 	{
 		pos=in.find_first_not_of(sep, pos);
 		if (pos==stmiss) // no more tokens
-			return out.size();
+			goto tok_done;
 		pos2=in.find_first_of(sep, pos);
 		if (pos2==stmiss) // no more terminators, EOL
 			pos2=oob;
 		out.push_back(in.substr(pos, pos2-pos));
 		pos=pos2+1;
 	}
-	return out.size();
+	tok_done:
+	return out.size()-nBefore;
 }
 
+
+void StrSubst(string &contents, const string &from, const string &to)
+{
+	tStrPos pos;
+	while (stmiss!=(pos=contents.find(from)))
+	{
+		contents.replace(pos, from.length(), to);
+		pos+=to.length();
+	}
+}
+
+
 void Join(MYSTD::string &out, const MYSTD::string & sep, const tStrVec & tokens)
 {
 	out.clear();
@@ -261,6 +279,12 @@ string tHttpUrl::ToString() const
 }
 //tHttpUrl::tHttpUrl() : pMisc(NULL) {};
 
+string ltos(long n)
+{
+	char buf[21];
+	snprintf(buf, 21, "%ld", n);
+	return buf;
+}
 
 /*
 const MYSTD::string GetDateString() {
